<?php
/**
 * WordPress Importer class for managing the import process of a WXR file
 *
 * @package WordPress
 * @subpackage Importer
 */

if ( class_exists( 'WP_Importer' ) && ! class_exists( 'Woo_Wallet_CSV_Importer' ) ) {

	/**
	 * CSV importer class.
	 */
	class Woo_Wallet_CSV_Importer extends WP_Importer {

		/**
		 * CSV column indexes.
		 *
		 * @var array
		 */
		public $column_indexes = array();
		/**
		 * CSV column keys.
		 *
		 * @var array
		 */
		public $column_keys = array();
		/**
		 * Class constructor.
		 */
		public function __construct() {
			add_action( 'admin_enqueue_scripts', array( $this, 'woo_wallet_csv_import_scripts' ), 10 );
		}
		/**
		 * Enqueue scripts.
		 */
		public function woo_wallet_csv_import_scripts() {
			$wp_scripts = wp_scripts();
			wp_register_style( 'woo-wallet-import-jquery-ui', '//ajax.googleapis.com/ajax/libs/jqueryui/' . $wp_scripts->registered['jquery-ui-core']->ver . '/themes/smoothness/jquery-ui.css', false, $wp_scripts->registered['jquery-ui-core']->ver, false );
			wp_register_script( 'woo_wallet_importer', woo_wallet_importer()->plugin_url() . '/assets/js/wallet-import.js', array( 'jquery' ), rand(), true );
		}

		/**
		 * Dispatcher function.
		 *
		 * @return void
		 */
		public function dispatch() {
			if ( empty( $_GET['step'] ) ) {
				$step = 0;
			} else {
				$step = (int) $_GET['step'];
			}
			$this->header( $step );
			switch ( $step ) {
				case 0:
					$this->greet();
					break;
				case 1:
					check_admin_referer( 'import-upload' );
					set_time_limit( 0 );
					echo '<div class="wc-progress-form-content woocommerce-importer woocommerce-importer__importing">';
					$result = $this->import();
					echo '</div>';
					if ( is_wp_error( $result ) ) {
						echo $result->get_error_message(); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					}
					break;
				case 2:
					$this->success();
					break;
			}

			$this->footer();
		}

		/**
		 * Importer header.
		 *
		 * @param int $step step.
		 * @return void
		 */
		public function header( $step ) {
			?>
			<style type="text/css">
				.woocommerce-exporter-wrapper .wc-progress-steps li, .woocommerce-importer-wrapper .wc-progress-steps li, .woocommerce-progress-form-wrapper .wc-progress-steps li{
					width: 33.3%;
				}
			</style>
			<div class="wrap woocommerce">
			<h1><?php esc_html_e( 'Import Wallet Transactions', 'woo-wallet-importer' ); ?></h1>
			<div class="woocommerce-progress-form-wrapper">
			<ol class="wc-progress-steps">
			<li class="<?php echo esc_attr( $step >= 0 ? 'active' : '' ); ?>"><?php esc_html_e( 'Upload CSV file', 'woo-wallet-importer' ); ?></li>
			<li class="<?php echo esc_attr( $step >= 1 ? 'active' : '' ); ?>"><?php esc_html_e( 'Import', 'woo-wallet-importer' ); ?></li>
			<li class="<?php echo esc_attr( $step >= 2 ? 'active' : '' ); ?>"><?php esc_html_e( 'Done!', 'woo-wallet-importer' ); ?></li>
			</ol>
			<?php
		}

		/**
		 * Importer page footer.
		 *
		 * @return void
		 */
		public function footer() {
			?>
			</div>
			</div>
			<?php
		}

		/**
		 * Step 1.
		 *
		 * @return void
		 */
		public function greet() {
			$action     = add_query_arg( 'step', 1 );
			$bytes      = apply_filters( 'import_upload_size_limit', wp_max_upload_size() );
			$size       = size_format( $bytes );
			$upload_dir = wp_upload_dir();
			if ( ! empty( $upload_dir['error'] ) ) :
				?>
				<div class="error"><p><?php esc_html_e( 'Before you can upload your import file, you will need to fix the following error:' ); ?></p>
					<p><strong><?php echo esc_html( $upload_dir['error'] ); ?></strong></p></div>
				<?php
			else :
				?>
				<form enctype="multipart/form-data" id="import-upload-form" method="post" class="wc-progress-form-content woocommerce-importer" action="<?php echo esc_url( wp_nonce_url( $action, 'import-upload' ) ); ?>">
					<header>
						<h2><?php esc_html_e( 'Import wallet transactions from a CSV file', 'woo-wallet-importer' ); ?></h2>
						<p><?php esc_html_e( 'This tool allows you to import wallet transactions to user accounts from a CSV file.', 'woo-wallet-importer' ); ?></p>
					</header>
					<section>
						<table class="form-table woocommerce-importer-options">
							<tbody>
								<tr>
									<th>
										<?php
										printf(
											'<label for="upload">%s</label>',
											esc_html__( 'Choose a file from your computer:', 'woo-wallet-importer' )
										);
										?>
									</th>
									<td>
										<input type="file" id="upload" name="import" size="25" />
										<br>
										<small><?php /* translators: %s: size */ echo sprintf( esc_html__( 'Maximum size: %s' ), $size ); ?></small>
										<br>
										<small><a download="sample.csv" href="<?php echo esc_url( untrailingslashit( plugins_url( '/', WOO_WALLET_IMPORTER_PLUGIN_FILE ) ) . '/sample.csv' ); ?>"><?php esc_html_e( 'Download Sample CSV', 'woo-wallet-importer' ); ?></a></small>
										<input type="hidden" name="action" value="save" />
										<input type="hidden" name="max_file_size" value="<?php echo esc_attr( $bytes ); ?>" />
									</td>
								</tr>
							</tbody>
						</table>
					</section>
					<div class="wc-actions">
						<?php submit_button( __( 'Upload file and import' ), 'primary' ); ?>
					</div>
				</form>
				<?php
			endif;
		}
		/**
		 * Success function
		 *
		 * @return void
		 */
		public function success() {
			?>
			<div class="wc-progress-form-content woocommerce-importer">
				<section class="woocommerce-importer-done">
					<?php esc_html_e( 'Import complete!', 'woo-wallet-importer' ); ?>
					<?php
					if ( isset( $_REQUEST['failed'] ) && $_REQUEST['failed'] ) { // phpcs:ignore WordPress.Security
						/* translators: %s: number of failed transactions */
						echo sprintf( __( 'Failed to import <strong>%s</strong> transactions.', 'woo-wallet-importer' ), $_REQUEST['failed'] ); // phpcs:ignore WordPress.Security
					}
					?>
					<a href="#" class="woo-wallet-importer-done-view-errors"><?php esc_html_e( 'View import log', 'woo-wallet-importer' ); ?></a>
				</section>

				<section class="woo-wallet-importer-error-log" style="display: none;padding-bottom: 24px;">
					<table class="widefat woo-wallet-importer-error-log-table">
						<thead>
							<tr>
								<th><?php esc_html_e( 'Transaction Logs', 'woo-wallet-importer' ); ?></th>
							</tr>
						</thead>
						<tbody>
							<?php
							$import_id = $_REQUEST['import_id']; // phpcs:ignore WordPress.Security
							$messages  = get_transient( "woo_wallet_import_messages_{$import_id}" );
							if ( $messages ) {
								foreach ( $messages as $message ) :
									?>
									<tr>
										<td><?php echo $message; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></td>
									</tr>
									<?php
								endforeach;
							}
							?>
						</tbody>
					</table>
				</section>
			</div>
			<script type="text/javascript">
				jQuery(function ($) {
					$('.woo-wallet-importer-done-view-errors').on('click', function (event) {
						event.preventDefault();
						$('.woo-wallet-importer-error-log').slideToggle();
					});
				});

			</script>
			<?php
		}
		/**
		 * Import function
		 *
		 * @return object | bool
		 */
		public function import() {
			$file = $this->wallet_import_handle_upload();

			if ( isset( $file['error'] ) ) {
				echo '<section><p><strong>' . esc_html__( 'Sorry, there has been an error.', 'woo-wallet-importer' ) . '</strong><br />';
				echo esc_html( $file['error'] ) . '</p></section>';
				return false;
			} elseif ( ! file_exists( $file['file'] ) ) {
				echo '<section><p><strong>' . esc_html__( 'Sorry, there has been an error.', 'woo-wallet-importer' ) . '</strong><br />';
				/* translators: %s: file path */
				printf( __( 'The export file could not be found at <code>%s</code>. It is likely that this was caused by a permissions problem.', 'woo-wallet-importer' ), esc_html( $file['file'] ) ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
				echo '</p></section>';
				return false;
			}

			$this->id   = (int) $file['id'];
			$this->file = get_attached_file( $this->id );
			if ( ! self::is_file_valid_csv( $this->file ) ) {
				echo '<section><p><strong>';
				esc_html_e( 'Invalid file type. The importer supports CSV and TXT file formats.', 'woo-wallet-importer' );
				echo '</p></section>';
				wp_import_cleanup( $this->id );
				return false;
			}
			$result = $this->process_posts();
			if ( is_wp_error( $result ) ) {
				return $result;
			}
		}
		/**
		 * Handle upload
		 *
		 * @return array
		 */
		private function wallet_import_handle_upload() {
			if ( ! isset( $_FILES['import'] ) ) {
				return array(
					'error' => sprintf(
							/* translators: 1: php.ini, 2: post_max_size, 3: upload_max_filesize */
						__( 'File is empty. Please upload something more substantial. This error could also be caused by uploads being disabled in your %1$s file or by %2$s being defined as smaller than %3$s in %1$s.' ),
						'php.ini',
						'post_max_size',
						'upload_max_filesize'
					),
				);
			}

			$overrides = array(
				'test_form' => false,
				'test_type' => false,
			);
			$upload    = wp_handle_upload( wc_clean( wp_unslash( $_FILES['import'] ) ), $overrides ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized

			if ( isset( $upload['error'] ) ) {
				return $upload;
			}

			// Construct the object array.
			$object = array(
				'post_title'     => wp_basename( $upload['file'] ),
				'post_content'   => $upload['url'],
				'post_mime_type' => $upload['type'],
				'guid'           => $upload['url'],
				'context'        => 'import',
				'post_status'    => 'private',
			);

			// Save the data.
			$id = wp_insert_attachment( $object, $upload['file'] );

			/*
			 * Schedule a cleanup for one day from now in case of failed
			 * import or missing wp_import_cleanup() call.
			 */
			wp_schedule_single_event( time() + DAY_IN_SECONDS, 'importer_scheduled_cleanup', array( $id ) );

			return array(
				'file' => $upload['file'],
				'id'   => $id,
			);
		}
		/**
		 * Check file type
		 *
		 * @param string $file file.
		 * @return bool
		 */
		public static function is_file_valid_csv( $file ) {
			$valid_filetypes = self::get_valid_csv_filetypes();
			$filetype        = wp_check_filetype( $file, $valid_filetypes );
			if ( in_array( $filetype['type'], $valid_filetypes, true ) ) {
				return true;
			}

			return false;
		}
		/**
		 * Get valid filetypes
		 *
		 * @return array
		 */
		protected static function get_valid_csv_filetypes() {
			return array(
				'csv' => 'text/csv',
				'txt' => 'text/plain',
			);
		}
		/**
		 * Process posts
		 *
		 * @return void
		 */
		public function process_posts() {
			wp_enqueue_style( 'woo-wallet-import-jquery-ui' );
			wp_enqueue_script( 'jquery-ui-progressbar' );
			wp_localize_script(
				'woo_wallet_importer',
				'woo_wallet_importer_param',
				array(
					'ajaxurl'      => admin_url( 'admin-ajax.php' ),
					'import_nonce' => wp_create_nonce( 'woo-wallet-import' ),
					'file'         => $this->file,
					'delimiter'    => WOO_WALLET_IMPORTER_DELIMITER,
					'import_id'    => $this->id,
				)
			);
			wp_enqueue_script( 'woo_wallet_importer' );
			?>
			<div class="woo-wallet-importer">
				<header>
					<span class="spinner is-active"></span>
					<h2><?php esc_html_e( 'Importing', 'woo-wallet-importer' ); ?></h2>
					<p><?php esc_html_e( 'Your transactions are now being imported...', 'woo-wallet-importer' ); ?></p>
				</header>
				<section style="padding: 16px;">
					<div class="woo-wallet-importer-progress"></div>
				</section>
			</div>
			<?php
		}

	}

}
